/* --8<--8<--8<--8<--
 *
 * Copyright (C) 2006 Smithsonian Astrophysical Observatory
 *
 * This file is part of mst_rdb
 *
 * mst_rdb is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * mst_rdb is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the 
 *       Free Software Foundation, Inc. 
 *       51 Franklin Street, Fifth Floor
 *       Boston, MA  02110-1301, USA
 *
 * -->8-->8-->8-->8-- */

/* rdb_is_column.c */

#include "mst_rdb.h"

#include <string.h>   /* for strcmp */

/*

  PROCEDURE
    rdb_is_column - Check whether /rdb header includes column "name".

  DESCRIPTION
    rdb_is_column compares the provided character string against the
    names of the /rdb columns.

  RETURNS
    The routine returns 1 if name is a column name, 0 otherwise.

  AUTHOR
    Terry Gaetz

*/
int
rdb_is_column(
  rdbHeader const *hdr,        /* the rdb file description */
  char const      *name        /* header name sought */
)
{
  int rc = 0;   /* assume name not included */
  int n;
  for ( n = 0 ; n < hdr->n ; ++n )
  {
    if (! strcmp( name, hdr->col_name[n] ) )
    {
      ++rc;
      break;
    }
  }
  return rc;
}
